/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.archive.impl;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.zip.ZipEntry;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.api.IArchiveMetadata;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.url.URLUtils;

/**
 * Creates wrapper around jar file.
 * @author Florent Benoit
 */
public class JarArchiveImpl extends AbsArchiveImpl implements IArchive {

    /**
     * Logger.
     */
    private static final Log logger = LogFactory.getLog(JarArchiveImpl.class);

    /**
     * Internal resource used as archive.
     */
    private JarFile jarFile = null;

    /**
     * Internal resource used as archive.
     */
    private File file = null;

    /**
     * Cached URL of this archive.
     */
    private URL url = null;

    /**
     * Metadata analyzed ?
     */
    private boolean metadataAnalyzed = false;

    /**
     * Creates new instance of an EZBArchive for a jar file.
     * @param file the given jar file.
     */
    protected JarArchiveImpl(final File file) {
        super();
        try {
            this.file = file.getCanonicalFile();
        } catch (IOException e) {
            logger.debug("File ''{0}'' cannot be canonicalized, use it as is", file);
            this.file = file;
        }
        this.url = URLUtils.fileToURL(this.file);
    }

    /**
     * Init an archive and create the associated jar file object. Close() method
     * needs to be called !.
     * @throws ArchiveException if init fails
     */
    private void init() throws ArchiveException {
        try {
            jarFile = new JarFile(file);
        } catch (IOException e) {
            throw new ArchiveException("Invalid file", e);
        }
    }

    /**
     * @return a description of this archive. This name could be used in logger
     *         info.
     */
    public String getName() {
        return file.getPath();
    }

    /**
     * Close the underlying Resource.
     * @return Returns <code>true</code> if the close was succesful,
     *         <code>false</code> otherwise.
     */
    public boolean close() {
        // Never accessed.
        if (jarFile == null) {
            return true;
        }
        try {
            jarFile.close();
        } catch (IOException e) {
            return false;
        }

        jarFile = null;
        return true;
    }

    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns the resource URL if the resource has been found. null
     *         otherwise.
     * @throws ArchiveException if method fails.
     */
    public URL getResource(final String resourceName) throws ArchiveException {
        URL resourceUrl = null;
        // Open jar file.
        init();
        try {
            JarEntry jarEntry = jarFile.getJarEntry(resourceName);
            if (jarEntry != null) {
                try {
                    resourceUrl = new URL("jar:" + url + "!/" + resourceName);
                } catch (MalformedURLException e) {
                    throw new ArchiveException("Invalid url", e);
                }
            }
        } finally {
            close();
        }
        return resourceUrl;
    }

    /**
     * @return Returns an Iterator of Resource's URL.
     * @throws ArchiveException if method fails.
     */
    public Iterator<URL> getResources() throws ArchiveException {
        List<URL> listResources = new ArrayList<URL>();
        init();
        try {
            Enumeration<? extends ZipEntry> en = jarFile.entries();
            while (en.hasMoreElements()) {
                ZipEntry zipEntry = en.nextElement();
                String name = zipEntry.getName();
                try {
                    listResources.add(new URL("jar:" + url + "!/" + name));
                } catch (MalformedURLException e) {
                    throw new ArchiveException("Invalid url", e);
                }
            }
        } finally {
            close();
        }
        return listResources.iterator();
    }

    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns an Iterator of matching resources.
     * @throws ArchiveException if method fails.
     */
    public Iterator<URL> getResources(final String resourceName) throws ArchiveException {
        List<URL> listResources = new ArrayList<URL>();
        URL url = getResource(resourceName);
        if (url != null) {
            listResources.add(url);
        }

        return listResources.iterator();
    }

    /**
     * @return Returns the resource URL.
     * @throws ArchiveException if method fails.
     */
    public URL getURL() throws ArchiveException {
        return this.url;
    }

    /**
     * Is that the given object is equals to our instance.
     * @param o the object to compare.
     * @return true if equals, else false.
     */
    @Override
    public boolean equals(final Object o) {
        if (!(o instanceof JarArchiveImpl)) {
            return false;
        }
        JarArchiveImpl other = (JarArchiveImpl) o;
        return this.file.equals(other.file);
    }

    /**
     * Gets hashcode for this object.
     * @return hash code.
     */
    @Override
    public int hashCode() {
        return file.hashCode();
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        return getName();
    }

    /**
     * Init metadata by reading the Manifest file.
     * @throws ArchiveException if metadata are not initialized
     */
    protected void initMetadata() throws ArchiveException {
        // Open Jar File
        init();

        // Get manifest
        Manifest manifest = null;
        try {
            manifest = jarFile.getManifest();
        } catch (IOException e) {
            throw new ArchiveException("Cannot analyze the manifest", e);
        } finally {
            // close jar
            close();
        }

        // Read manifest
        if (manifest != null) {
            readManifest(manifest);
        }

    }

    /**
     * @return the Metadata associated to this archive.
     */
    @Override
    public IArchiveMetadata getMetadata() {
        if (!metadataAnalyzed) {
            try {
                initMetadata();
            } catch (ArchiveException e) {
                logger.error("Cannot analyze metadata for archive " + jarFile, e);
            }
            metadataAnalyzed = true;
        }
        return super.getMetadata();

    }

    /**
     * @return all resources name
     * @throws ArchiveException if error occurs
     */
    public Iterator<String> getEntries() throws ArchiveException {
        List<String> listResourcesName = new ArrayList<String>();
        init();
        try {
            Enumeration<? extends ZipEntry> en = jarFile.entries();
            while (en.hasMoreElements()) {
                ZipEntry zipEntry = en.nextElement();
                String name = zipEntry.getName();
                listResourcesName.add(name);
            }
        } finally {
            close();
        }
        return listResourcesName.iterator();
    }

}
